@@ -1,6 +1,9 @@
 import os
 import yaml
 
+from jinja2 import Template
+from io import StringIO
+
 try:
     from yaml import CLoader as base_loader, CDumper as Dumper
 except ImportError:
@@ -16,7 +19,10 @@ Accessed 2020-07-28
 class Loader(base_loader):
     def __init__(self, stream):
         self._root = os.path.split(stream.name)[0]
-        super().__init__(stream)
+        # Enable Jinja2 in the yaml files
+        yaml_stream = StringIO(Template(stream.read()).render())
+        yaml_stream.name = stream.name
+        super().__init__(yaml_stream)
 
     def scalar_from_file(self, node):
         filename = os.path.join(self._root, self.construct_scalar(node))
