@@ -9,12 +9,18 @@ from ewah.hooks.base import EWAHBaseHook
 from ewah.uploaders import get_uploader
 
 from datetime import datetime, timedelta
-from typing import Optional, List, Dict
+from tempfile import TemporaryFile, TemporaryDirectory
+from typing import Optional, List, Dict, Union
 
+import bz2
 import copy
+import gzip
 import hashlib
-import time
+import lzma
+import os
+import pickle
 import sys
+import time
 
 
 class EWAHBaseOperator(BaseOperator):
@@ -102,7 +108,7 @@ class EWAHBaseOperator(BaseOperator):
             exc_type, exc_obj, exc_tb = sys.exc_info()
             try:
                 dag_id = (dag or self.dag).dag_id
-            except Exception as e:
+            except Exception as ex:
                 dag_id = "[unknown]"
             raise Exception(
                 """
@@ -168,14 +174,28 @@ class EWAHBaseOperator(BaseOperator):
         add_metadata=True,
         rename_columns: Optional[Dict[str, str]] = None,  # Rename columns
         subsequent_field=None,  # field name to use for subsequent extract strategy
+        default_timezone=None,  # specify a default time zone for tz-naive datetimes
+        use_temp_pickling=True,  # use new upload method if True - use it by default
+        pickling_upload_chunk_size=100000,  # default chunk size for pickle upload
+        pickle_compression=None,  # data compression algorithm to use for pickles
         *args,
         **kwargs
     ):
         super().__init__(*args, **kwargs)
 
+        assert pickle_compression is None or pickle_compression in (
+            "gzip",
+            "bz2",
+            "lzma",
+        )
+
         assert not (rename_columns and columns_definition)
         assert rename_columns is None or isinstance(rename_columns, dict)
 
+        if default_timezone:
+            assert dwh_engine in (EC.DWH_ENGINE_POSTGRES,)  # Only for PostgreSQL so far
+            assert not ";" in default_timezone  # Avoid SQL Injection
+
         # Check if the extract and load strategies are allowed in combination
         # Also check if required params are supplied for the strategies
         if extract_strategy == EC.ES_SUBSEQUENT:
@@ -328,6 +348,10 @@ class EWAHBaseOperator(BaseOperator):
         self.add_metadata = add_metadata
         self.rename_columns = rename_columns
         self.subsequent_field = subsequent_field
+        self.default_timezone = default_timezone
+        self.use_temp_pickling = use_temp_pickling
+        self.pickling_upload_chunk_size = pickling_upload_chunk_size
+        self.pickle_compression = pickle_compression
 
         self.uploader = get_uploader(self.dwh_engine)
 
@@ -336,6 +360,19 @@ class EWAHBaseOperator(BaseOperator):
         )
         # assert self.uploader._ACCEPTED_EXTRACT_STRATEGIES.get(extract_strategy), _msg
 
+    @staticmethod
+    def _pickle_file_open(name, mode, compression_mode):
+        if compression_mode is None:
+            return open(name, mode)
+        elif compression_mode == "gzip":
+            return gzip.open(name, mode)
+        elif compression_mode == "lzma":
+            return lzma.open(name, mode)
+        elif compression_mode == "bz2":
+            return bz2.open(name, mode)
+        else:
+            raise Exception("Compression Mode not implemented!")
+
     def ewah_execute(self, context):
         raise Exception("You need to overwrite me!")
 
@@ -371,6 +408,12 @@ class EWAHBaseOperator(BaseOperator):
 
         self.uploader = self.uploader(EWAHBaseHook.get_connection(self.dwh_conn_id))
 
+        # If applicable: set the session's default time zone
+        if self.default_timezone:
+            self.uploader.dwh_hook.execute(
+                "SET timezone TO '{0}'".format(self.default_timezone)
+            )
+
         if self.source_conn_id:
             # resolve conn id here & delete the object to avoid usage elsewhere
             self.source_conn = EWAHBaseHook.get_connection(self.source_conn_id)
@@ -431,6 +474,14 @@ class EWAHBaseOperator(BaseOperator):
         del self.load_data_from_relative
         del self.load_data_until_relative
 
+        # Prepare file used for temporary data pickling, if applicable
+        if self.use_temp_pickling:
+            temp_pickle_folder = TemporaryDirectory()
+            temp_file_name = temp_pickle_folder.name + os.sep + "temp_pickle_file"
+            self.temp_pickle_file = self._pickle_file_open(
+                temp_file_name, "wb", self.pickle_compression
+            )
+
         # Have an option to wait until a short period (e.g. 2 minutes) past
         # the incremental loading range timeframe to ensure that all data is
         # loaded, useful e.g. if APIs lag or if server timestamps are not
@@ -468,6 +519,29 @@ class EWAHBaseOperator(BaseOperator):
         else:
             self.ewah_execute(context)
 
+        if self.use_temp_pickling:
+            # then upload data now
+            self.temp_pickle_file.close()
+            self.temp_pickle_file = self._pickle_file_open(
+                temp_file_name, "rb", self.pickle_compression
+            )
+            keep_unpickling = True
+            while keep_unpickling:
+                raw_data = []
+                for _ in range(self.pickling_upload_chunk_size):
+                    try:
+                        raw_data.append(pickle.load(self.temp_pickle_file))
+                    except EOFError:
+                        # all done
+                        keep_unpickling = False
+                        break
+                self._upload_data(raw_data)
+
+            # always clean up after yourself
+            self.temp_pickle_file.close()
+            del self.temp_pickle_file
+            del temp_pickle_folder
+
         # if PostgreSQL and arg given: create indices
         for column in self.index_columns:
             assert self.dwh_engine == EC.DWH_ENGINE_POSTGRES
@@ -583,7 +657,31 @@ class EWAHBaseOperator(BaseOperator):
                         )
         return result
 
+    def _upload_via_pickling(self, data: Union[dict, List[dict]]):
+        """Call this function to earmark a dictionary for later upload."""
+        assert self.use_temp_pickling, "Can only call function if using temp pickling!"
+        if isinstance(data, dict):
+            pickle.dump(data, self.temp_pickle_file)
+        elif isinstance(data, list):
+            self.log.info(
+                "Pickling {0} rows of data for later upload...".format(len(data))
+            )
+            for row in data:
+                self._upload_via_pickling(row)
+        else:
+            raise Exception(
+                "Invalid data format for function! Must be dict or list of dicts!"
+            )
+
     def upload_data(self, data=None, columns_definition=None):
+        if self.use_temp_pickling:
+            # earmark for later upload
+            self._upload_via_pickling(data=data)
+        else:
+            # upload straightaway
+            self._upload_data(data, columns_definition)
+
+    def _upload_data(self, data=None, columns_definition=None):
         """Upload data, no matter the source. Call this functions in the child
         operator whenever data is available for upload, as often as needed.
         """
